// A modified version for this project: https://github.com/zhangbozhb/WebViewBridge.Swift

var MagpieBridge = window.MagpieBridge || {};
window.MagpieBridge = MagpieBridge;

MagpieBridge.Core = MagpieBridge.Core || (function() {
    var callbackId = 1;
    var callbacks = {};
    var actionQueue = [];
    var handlerMapper = {};

    var createBridge = function() {
       /* var iFrame;
        iFrame = document.createElement("iframe");
        iFrame.setAttribute("src", "MagpieBridge://__BRIDGE_LOADED__");
        iFrame.setAttribute("style", "display:none;");
        iFrame.setAttribute("height", "0px");
        iFrame.setAttribute("width", "0px");
        iFrame.setAttribute("frameborder", "0");
        document.body.appendChild(iFrame);
        setTimeout(function() {
            document.body.removeChild(iFrame)
        }, 0);*/
    };

    var dispatchAction = function() {
        var actionName = arguments[0];
        var actionArgs = arguments[1] || {};
        var actionCallback = arguments[2];
        var actionId = ++callbackId;

        if (actionCallback) {
            callbacks[actionId] = actionCallback;
        } else {
            actionId = 0;
        }

        var action = {
            id: actionId,
            name: actionName,
            data: actionArgs
        };
        if (window.webkit && window.webkit.messageHandlers &&
            window.webkit.messageHandlers.MagpieBridge &&
            window.webkit.messageHandlers.MagpieBridge.postMessage) {
            window.webkit.messageHandlers.MagpieBridge.postMessage(JSON.stringify([action]))
        } else if (window.yamibuy_messageHandlers &&
            window.yamibuy_messageHandlers.MagpieBridge &&
            window.yamibuy_messageHandlers.MagpieBridge.postMessage) {
            window.yamibuy_messageHandlers.MagpieBridge.postMessage(JSON.stringify([action]))
        } else {
            actionQueue.push(action);
            createBridge();
        }

        if (window.yamibuy_android_bridge && window.yamibuy_android_bridge.handleAction) {
            window.yamibuy_android_bridge.handleAction(JSON.stringify([action]));
        }
    };

    var callbackResult = function() {
        var data = arguments[0];
        if (!data) {
            return
        }
        var callInfo = JSON.parse(data);
        var callbackId = callInfo["id"];
        var status = callInfo["status"];
        var data = callInfo["result"];
        if (!callbackId || status == undefined || data == undefined) {
            return
        }
        var callback = callbacks[callbackId];
        if (!callback) {
            return
        }
        var result = callback(status, data);
        return result != undefined ? JSON.stringify(result) : undefined;
    };

    var completeActions = function() {
        var json = JSON.stringify(actionQueue);
        actionQueue = [];
        return json;
    };

    var registerJSHandler = function(handlerName, callback) {
        handlerMapper[handlerName] = callback
    };

    var callJSHandler = function(data) {
        var callInfo = data;
        var name = callInfo["name"];
        var args = callInfo["args"];
        var argsCount = callInfo["argsCount"];
        if (!name || argsCount == undefined || argsCount != args.length) {
            return;
        }

        var handler = handlerMapper[name];
        if (handler) {
            var result = handler.apply(this, args);
            return result != undefined ? JSON.stringify(result) : undefined
        }
    };

    var ready = function() {
        var readyFunction = arguments[0];
        if (readyFunction) {
            document.addEventListener("DOMContentLoaded", readyFunction);
        }
    };

    return {
        // For Script Side
        dispatchAction: dispatchAction,
        registerJSHandler: registerJSHandler,

        // For Native Side
        callbackResult: callbackResult,
        callJSHandler: callJSHandler,
        completeActions: completeActions,

        ready: ready
    }
}());


MagpieBridge.Core.ready(function() {
    var appVer = window.YamibuyAppVersion ? window.YamibuyAppVersion.split("@") : []

    window.$YamibuyRefreshAppVersion = function() {
        appVer = window.YamibuyAppVersion ? window.YamibuyAppVersion.split("@") : []
    }

    var compareVersion = function(vA, vB) {
        if (vA === vB) {
            return 0;
        }

        var partsNumberA = vA.split(".");
        var partsNumberB = vB.split(".");

        for (var i = 0; i < partsNumberA.length; i++) {
            var valueA = parseInt(partsNumberA[i]);
            var valueB = parseInt(partsNumberB[i]);
            // A bigger than B
            if (valueA > valueB || isNaN(valueB)) {
                return 1;
            }
            // B bigger than A
            if (valueA < valueB) {
                return -1;
            }
        }
    }

    var isIOSDevice = function() {
        if (appVer.length != 2) {
            return false
        }
        return appVer[0] == "iOS"
    }

    var provisionedIOS = function() {
        if (appVer.length != 2) {
            return false
        }
        if (appVer[0] == "iOS" && compareVersion(appVer[1], "3.2.7") > 0) {
            return true
        }
        return false
    }

    var isAndroidDevice = function() {
        if (appVer.length != 2) {
            return false
        }
        return appVer[0] == "Android"
    }

    var provisionedAndroid = function() {
        if (appVer.length != 2) {
            return false
        }
        if (appVer[0] == "Android" && compareVersion(appVer[1], "47") > 0) {
            return true
        }
        return false
    }

    var useNewApi = function() {
        return provisionedIOS() || provisionedAndroid()
    }

    MagpieBridge.Yamibuy = MagpieBridge.Yamibuy || (function() {
        var queryToken = function(done) {
            if (useNewApi()) {
                MagpieBridge.Core.dispatchAction(
                    "User.QueryToken", { callback: done.name },
                    function(status, data) {});
            } else {
                window.location.href = "ymb://queryToken?callback=" + done.name
            }
        };

        var checkLogin = function(done) {
            if (useNewApi()) {
                MagpieBridge.Core.dispatchAction(
                    "User.CheckLogin", { callback: done.name },
                    function(status, data) {});
            } else {
                window.location.href = "ymb://isLogin?callback=" + done.name
            }
        };

        var refreshZipcode = function(done) {
            //if (useNewApi()) {
                MagpieBridge.Core.dispatchAction(
                    "User.RefreshZipcode",
                    function(status, data) {});
         //   }
        };

        var finishPayment = function(purchaseId, done) {
            MagpieBridge.Core.dispatchAction(
                "Order.FinishPayment", { purchase_id: purchaseId },
                function(status, data) {
                    status ? done(data) : done("")
                });
        };

        var checkActionReady = function(name, done) {
            MagpieBridge.Core.dispatchAction(
                "Core.CheckReady", { action: name },
                function(status, result) {
                    if (done) { done(result) }
                });
        }

        var getStatusBarHeight = function() {
            MagpieBridge.Core.dispatchAction(
                "Core.GetStatusBarHeight",
                function(status, result) {
                    if (result) { return result }
                });
                return result;
        }

        var addToCart = function(goodsId, campaignId, done) {
            var action = "Cart.AddItem"
            var actionData = {
                goods_id: goodsId,
                campaign_id: campaignId
            }
            if (useNewApi()) {
                MagpieBridge.Core.dispatchAction(
                    action, actionData,
                    function(status, result) {
                        if (done) { done(status) }
                    });
            } else {
                window.location.href = "ymb://addToCart?param0=" + goodsId
            }
        }

        var addToCartInBatch = function(goodsIds, done) {
            var action = "Cart.BatchAddItems"
            if (useNewApi()) {
                MagpieBridge.Core.dispatchAction(
                    action, { goods_ids: goodsIds },
                    function(status, result) {
                        if (done) { done(status) }
                    });
            } else {
                window.location.href = "ymb://batchAddToCart?param0=" + goodsIds
            }
        }

        var shareContent = function(url, title, description, image, mode, done) {
            var action = "Misc.Share"
            var actionData = {
                url: url,
                title: title,
                description: description,
                image: image,
                mode: mode
            }
            if (useNewApi()) {
                MagpieBridge.Core.dispatchAction(
                    action, actionData,
                    function(status, result) {
                        if (done) { done(status) }
                    });
            } else {
                window.location.href = "ymb://share?param0=" + encodeURIComponent(url) + "&param1=" + encodeURIComponent(title) + "&param2=" + image + "&param3=" + encodeURIComponent(description)
            }
        }

        var openLink = function(url, title, done) {
            var action = "Misc.OpenLink"
            var actionData = {
                url: url,
                title: title
            }
            if (useNewApi()) {
                MagpieBridge.Core.dispatchAction(
                    action, actionData,
                    function(status, result) {
                        if (done) { done(status) }
                    });
            } else {
                window.location.href = "ymb://openURL?url=" + encodeURIComponent(url) + "&title=" + encodeURIComponent(title)
            }
        }

        return {
            queryToken: queryToken,
            checkLogin: checkLogin,
            finishPayment: finishPayment,
            addToCart: addToCart,
            addToCartInBatch: addToCartInBatch,
            shareContent: shareContent,
            openLink: openLink,
            getStatusBarHeight: getStatusBarHeight,
            refreshZipcode: refreshZipcode,
        }
    }());
});
